function R = CovarExpDecay(N,phi1,phi2,phi3,C)
% Bivariate exponentially decaying covariance (toy-example)
%
% Usage:
%   R = CovarExpDecay(N,phi1,phi2,phi3,C)
% Inputs:
%   N     length of the covariance sequence
%   phi1  parameter for auto-covariance of 1st coordinate
%   phi2  parameter for auto-covariance of 2nd coordinate
%   phi3  parameter for the cross-covariance
%   C     parameter for the cross-covariance
% Outputs:
%   R     generated covariance structure, a cell array of size 2-by-2
%         R{1,1}=r1, R{2,2}=r2, R{1,2}=r12, where
%           r1    auto-covariance for 1st coordinate
%           r2    auto-covariance for 2nd coordinate
%           r12   cross-covariance for 1st and 2nd coordinate
%                 The outputs are row-vectors 
%                 with the values r1(h),r2(h),r12(h) for h=0,1,...,N-1
%
% Comments:
%   The auto-covariance sequences:
%     r1(h)=phi1^h, 0<phi1<1
%     r2(h)=phi2^h, 0<phi2<1
%   The cross-covariance sequence:
%     r12(h)=C*phi3^h, with
%     0<C<1, |phi3|<phi1,phi2
%     C|1-phi3| < 1-phi1, 1-phi2
%     C|1-phi3|^2 < |1-phi1|^2, |1-phi2|^2
%
%   Note: The cross-covariance is assumed to be symmetric, i.e.,
%   the process is time-reversible.
%
%
% Copyright 2009, Hannes Helgason, ENS Lyon

% Check conditions on parameters
if ~(phi1>0 && phi1<1),
    error('The parameter phi1 does not satisfy 0<phi1<1');
end
if ~(phi2>0 && phi2<1),
    error('The parameter phi2 does not satisfy 0<phi2<1');
end
if ~(C>0 && C<1),
    error('The parameter C does not satisfy 0<C<1');
end
if abs(phi3)>=phi1,
    error('The parameter phi3 does not satisfy |phi3|<phi1');
end
if abs(phi3)>=phi2,
    error('The parameter phi3 does not satisfy |phi3|<phi2');
end
tmp = C*(1-phi3); 
if tmp>=1-phi1,
    error(sprintf('The inequality C|1-phi3| < 1-phi1 is not satisfied\nC|1-phi3|=%0.4f, 1-phi1=%0.4f',tmp,1-phi1));
end
if tmp>=1-phi2,
    error(sprintf('The inequality C|1-phi3| < 1-phi2 is not satisfied\nC|1-phi3|=%0.4f, 1-phi2=%0.4f',tmp,1-phi2));
end
tmp = C*(1-phi3)^2; 
if tmp>=abs(1-phi1)^2,
    error(sprintf('The inequality C|1-phi3|^2 < |1-phi1|^2 is not satisfied\nC|1-phi3|^2=%0.4f, |1-phi1|^2=%0.4f',tmp,(1-phi1)^2));
end
if tmp>=abs(1-phi2)^2,
    error(sprintf('The inequality C|1-phi3|^2 < |1-phi2|^2 is not satisfied\nC|1-phi3|^2=%0.4f, |1-phi2|^2=%0.4f',tmp,(1-phi2)^2));    
end

% Generate the sequences
h = 0:(N-1);
r1 = phi1.^h;
r2 = phi2.^h;
r12 = C*phi3.^h;

P = 2;
R = cell(P,P);
R{1,1} = r1;
R{2,2} = r2;
R{1,2} = r12;

