function [X1,Y1,X2,Y2] = GenBivarGauss(R,N,genTR)
% Generate bivariate stationary Gaussian process
% W[n]=(X[n],Y[n]), n=0,1,...,N-1, with E(W[n]) = (0,0)
%
% Usage:
%  [X1,Y1,X2,Y2] = GenBivarGauss(R,N,genTR)
%
% Inputs:
%  R    Covariance structure stored as a 2-by-2 cell array:
%         R{1,1}= Rx   
%               auto-covariance sequence for the x-coordinate,
%               i.e., the sequence r_X[k] = E(X[k]X[0]), k=0,...,N'-1
%               The vector indexing is: Rx(k)=r_X[k-1]
%         R{2,2}=Ry
%               auto-covariance sequence for the y-coordinate
%               i.e., the sequence r_Y[k] = E(Y[k]Y[0]), k=0,...,N'-1
%               The vector indexing is: Ry(k)=r_Y[k-1]
%         R{1,2}=Rxy  
%               cross-covariance sequence
%               i.e., the sequence r_XY[k] = E(X[k]Y[0]), k=0,...,N'-1
%               The vector indexing is: Rxy(k)=r_XY[k-1]
%         R{2,1}=Ryx  
%               cross-covariance sequence
%               i.e., the sequence r_YX[k] = E(Y[k]X[0]), k=0,...,N'-1
%               The vector indexing is: Ryx(k)=r_YX[k-1]
%        REMARKS: 
%           * If R{2,1} is empty, the code assumes r_YX = r_XY
%           * For symmetric cross-covariance (time reversibility) N'=N
%           * For asymmetric cross-covariance N'=N+1
%  N     The length of the synthesized process
%  genTR -a flag set to "true" to force the code to generate a 
%        time-reverible process, i.e., assumes r_YX = r_XY. In this case, 
%        only the cells R{1,1}, R{2,2}, and R{1,2} will be used.
%        -If the flag is omitted or set to "false", the code will use
%        R{2,1} for r_YX (if it is not empty)
% 
% Outputs:
%   (X1,Y1) and (X2,Y2) are two independent realizations of W=(X,Y), 
%
% Copyright (c) Hannes Helgason, ENS Lyon, 2009


if nargin==3,
    if ~genTR & isempty(R{2,1}),
        warning('genTR is set to false but R{2,1} is empty. Will assume symmetric cross-covariance.'); 
    end
elseif nargin<3,
    genTR = false;
end

if genTR | isempty(R{2,1}),
    % Using symmetric cross-covariance (time reversibility)
    asymm = false;
else
    asymm = true;
end

% circular embedding of covariance
Rcirc = circembed(R,asymm);

% initialize (calculate auto- and cross-spectrum)
[sqrtLx,sqrtLy,Sxy] = InitBivarGauss(Rcirc);
    
% generate the process
[X1,Y1,X2,Y2] = SynthStepBivarGauss(sqrtLx,sqrtLy,Sxy,N);
