function [X,Xiid,Rapprox,approxFlag,BmArray] = GenMultivarGaussBestApprox(R,N,genTR)
% Generate multivariate stationary Gaussian process
% (X_1[n],...,X_P[n]), n=0,1,...,N-1, with E(W[n]) = (0,0)
%
% Uses the alternative construction (\tilde{X}=F^* U) introduced
% in the multivariate non-Gaussian synthesis paper.
%
% Usage:
%  [X,Xiid,Rapprox,approxFlag,BmArray] = GenMultivarGaussBestApprox(R,N,genTR)
%  [X,Xiid,Rapprox,approxFlag,BmArray] = GenMultivarGaussBestApprox(R,N)
%
% Inputs:
%   R    a cell structure of size P-by-P, i.e.,
%        initialized with the command "R=cell(P,P)" 
% 
%        The auto-covariance sequence for coordinate k, 
%        i.e., the sequence R_Xk[n] = E(Xk[n]Xk[0]), n=0,...,N-1
%        should be stored as a row vector in cell (k,k), i.e., in R{k,k}
%        Note, the vector indexing is: R{k,k}(m)=R_Xk[m-1]
% 
%        The cross-covariance sequence for coordinates k,j, with 1<=k<j<=P, 
%        i.e., the sequence R_XkXj[n] = E(Xk[n]Xj[0]), n=0,...,N-1
%        should be stored as a row vector in cell (k,j), i.e., in R{k,j}
%        Note, the vector indexing is: R{k,j}(m)=R_XkXj[m-1]
% 
%        All the auto- and cross-covariance sequences should have equal
%        number of entries.
%        NOTE: The cell entries R{k,j} with k>j, should be left empty 
% 
%        Example:
%        Let Rx,Ry,Rxy be vectors storing auto- and cross-covariance
%        sequences. Then we initialize the cell structure this way:
%        >> R = cell(2,2);
%        >> R{1,1} = Rx;
%        >> R{2,2} = Ry;
%        >> R{1,2} = Rxy;
%
%  N     The length of the synthesized process
%  genTR -a flag set to "true" to force the code to generate a 
%        time-reverible process, i.e., assumes r_YX = r_XY. In this case, 
%        only the cells R{k,j}, k<=j, will be used.
%        -If the flag is omitted or set to "false", the code will use
%        R{j,k} for r_YX (if it is not empty or not)
%
% Outputs:
%   X and Xiid are two independent realizations of the process, stored
%   as matrices of size P-by-N. X(k,:) is a realization of Xk, i.e.,
%   component k, and is a vector of length N. Same holds for Xiid.
%
%   Rapprox     resulting covariance structure (approximated if R is not a
%               valid covariance structure after circulant embedding)
%
%   approxFlag  true if approximation was needed, otherwise false
%
%   BmArray     this structure can be passed to
%               SynthStepMultivarGaussBestApprox to generate samples of
%               the process, e.g., for Monte Carlo simulations
%
% Copyright (c) Hannes Helgason, ENS Lyon, 2010

% set default value
if nargin<3,
	genTR = false;
end

% initalize
[Rapprox,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(R,N,genTR);

% generate the series
[X,Xiid] = SynthStepMultivarGaussBestApprox(BmArray,N);
