function [w,x] = GHermiteRule(N)
% Gauss-Hermite quadrature calcuation
%
% Usage:
%   [w,x] = GHermiteRule(N)
% Inputs and Outputs:
%   Given N, this routine returns arrays x, w containing
%   the abscissas and weights of the N-point Gauss-Hermite quadrature formula
%   The largest abscissa is returned in x(1), the most negative in x(N)
%
% Copyright (c), Hannes Helgason, ENS Lyon, 2009

%   Note: This M-file is intended to document the algorithm.
%   If a MEX file for a particular architecture exists,
%   it will be executed instead, but its functionality is the same.

% This function is based on Chapter 4 in Numerical Recipes in C

RELPREC = 1e-14; % relative precision, stopping criteria for Newton's method
MAXITER = 12;    % maximum number of iterations in Newton's method
PICONST = 1/(pi)^(1/4);

% Roots are symmetric about the origin, only need to find half of them
m = floor((N+1)/2);

for k=1:m,
    % find root k
    
    % Initial guesses
    if k==1,
        z = sqrt(2*N+1)-1.85575*(2*N+1)^(-1/6);
    elseif k==2,
        z = z-1.14*N^0.426/z;
    elseif k==3,
        z = 1.86*z-0.86*x(1);
    elseif k==4,
        z = 1.91*z-0.91*x(2);
    else
        z = 2*z-x(k-2);
    end

    % Refinment of guess using Newton's method
    for iter=1:MAXITER,
        % use recurrance relation to evaluate Hermite polynomial at z
        p1 = PICONST;
        p2 = 0;
        for n=1:N,
            p3=p2;
            p2=p1;
            p1=z*sqrt(2/n)*p2-sqrt((n-1)/n)*p3;
        end
        % calculate the derivative
        pp=sqrt(2*N)*p2;
        
        % Newton's formula
        z1=z;
        z=z1-p1/pp;
        if abs(z-z1)<=RELPREC,
            break;
        end
    end
    if iter==MAXITER,
        warning(sprintf('Reached maximum number of iterations in root finding! Relative error: %0.8f',abs(z-z1)));
    end
    % store the root and its symmetric counterpart
    x(k) = z;
    x(N+1-k) = -z;
    % compute the weights
    w(k) = 2/(pp*pp);
    w(N+1-k) = w(k);
end