function [RYapprox,approxFlag,BmArray,coeffs,RXapprox] = InitMVnGaussUsingHermite(RY,funchArray,N,M,xbound,zeropadLength)
% Initialization routine for multivariate non-Gaussian synthesis
% using Hermite expansions
%
% Usage:
%   [RYapprox,approxFlag,BmArray,coeffs,RXapprox] =
%   InitMVnGaussUsingHermite(RY,funchArray,N,M,xbound,zeropadLength)
% Inputs:
%   RY      The targeted covariance of non-Gaussian series.
%           NOTE: This sequence will be normalized to match the variance
%           of the marginal set by funchArray
%           The length of the sequences has to be greater than N
%   funchArray   
%           cell array of function handles for the non-linear transform 
%           f_p in Y_p=f_p(X_p)
%           funchArray{p} corresponds to coordinate p
%           Example: funchArray{p} = @(x) chi2cdf(normcdf(x),1), 
%                    This will make Y_p chi-squared with one degree of freedom           
%   N       The desired length of the synthesized series
%           Does not need to be equal to the length of the covariance
%           sequence
%   M       The order of the Hermite expansion to use for calculating 
%           the covariance relationship, default value is 15
%   xbound  for configuring the interval used in calculating the Hermite coefficients,
%           if omitted, the default value of HermiteCoeffExpansion will be used
%   zeropadLength  use this to increase the number of terms in the reversion series, 
%           if omitted, default value is zero
%PEND: Is zeropadLength really needed?
% Outputs:
%   RYapprox resulting covariance of the synthesized non-Gaussian
%            series
%   BmArray  this structure can be passed to
%            SynthStepMultivarGaussBestApprox to generate samples of
%            the Gaussian process which is then transformed
%   approxFlag  true if approximation was needed, otherwise false
%   coeffs   numerically calculated Hermite coefficients for the functions 
%            in funchArray
%   RXapprox the covariance sequence used in the synthesis of the Gaussian
%            series X and gives the covariance RYapprox
%
%  NOTE: 
%    The targeted covariance RY{k,j} will be renormalized by a factor 
%       sqrt(Var(Y_k)/RY{k,k}(1))*sqrt(Var(Y_j)/RY{j,j}(1)),
%    where Y_p=f_p(X) and RY{p,p}(1) is the inputted variance for
%    coordinate p
%
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon

if nargin<6,
  zeropadLength = 0;
  if nargin<4,
    M = 15;
  end
end  

P = size(RY,1); % number of coordinates

coeffs = cell(1,P);
% Calculate the Hermite expansion of the function
if nargin<5,
	% xbound is not set
    if iscell(funchArray),
        for k=1:P,
            funch = funchArray{k};
            coeffs{k} = HermiteCoeffExpansion(funch,M);
        end 
    else
        % using the same function handle for every coordinate
        funch = funchArray{k};
        coeffs{1} = HermiteCoeffExpansion(funch,M);
        for k=2:P,
            coeffs{k} = coeffs{1};
        end 
    end
else
	% xbound is set
    if iscell(funchArray),
        for k=1:P,
            funch = funchArray{k};
            coeffs{k} = HermiteCoeffExpansion(funch,M,xbound);
        end 
    else
        % using the same function handle for every coordinate
        funch = funchArray{k};
        coeffs{1} = HermiteCoeffExpansion(funch,M,xbound);
        for k=2:P,
            coeffs{k} = coeffs{1};
        end 
    end
end

% Calculate the coefficients in the forward covariance relationship,
%   rY= \sum_{m=1}^M fwdSeq(m)*rX^m
% and in the reversion series, which can be used to invert the above relationship,
% i.e.,
%   rX \approx \sum_{m=1}^M revSeq(m)*rY^m
% NOTE! This series does not necessarily give the inverse for all rY, but for
% some neighborhood around rY=0
% NOTE: The current version of the code requires c_1~=0, i.e., coeffs(2)~=0
fwdPolyCoeff = cell(P,P);
revPolyCoeff = cell(P,P);
normFactor = zeros(1,P);
for k=1:P,
    for j=k:P,
        [fwdSeq, revSeq] = CovarSequence(coeffs{k}, coeffs{j}, zeropadLength);
        % Map rY to rX
        fwdPolyCoeff{k,j} = fliplr([0 fwdSeq]);
        revPolyCoeff{k,j} = fliplr([0 revSeq]);
    end
    % Find the variance of the marginal for normalization
    % we get the variance by plugging rX=1
    varY = polyval(fwdPolyCoeff{k,k},1);
    normFactor(k) = sqrt(varY/RY{k,k}(1));
end

% Normalize targeted covariance and invert to get pseudo-covariance for
% the Gaussian sequence
RX = cell(P,P);
RYnormalized = cell(P,P);
thresh = 1e-4; %PEND! Make this parameter adjustable...
for k=1:P,
    for j=1:P,
        if ~isempty(RY{k,j}),
            % normalize the input covariance so it matches the functions in
            % funchArray
            rYnormalized = normFactor(k)*normFactor(j)*RY{k,j};
            % Do the reversion
            lowInd = min(k,j);
            highInd = max(k,j);
            rX = polyval(revPolyCoeff{lowInd,highInd},rYnormalized);
            if j==k,
                % forcing the variance in the Gaussian sequence to be 1
                rX(1) = 1; 
            end
            % Go back to see where reversion was not good
            rYreversion = polyval(fwdPolyCoeff{lowInd,highInd},rX);
            indBAD = find(abs(rYnormalized-rYreversion)>thresh);
            for ind=indBAD,
                rX2rY = @(t) polyval(fwdPolyCoeff{lowInd,highInd},t)-rYnormalized(ind);
                rX(ind) = fzero(rX2rY,1);
            end
            % store pseudo-covariance
            RX{k,j} = rX;
        end
    end
end
           
% perform initialization step for multivariate Gaussian synthesis
[RXapprox,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(RX,N);
%[RXapprox,approxFlag,BmArray] = InitMGBestApproxLapack(RX,N); % faster routine using LAPACK

% Map RXapprox to get the resulting covariance of the synthesized 
% non-Gaussian series
for k=1:P,
    for j=1:P,
        if ~isempty(RXapprox{k,j}),
            RYapprox{k,j} = polyval(fwdPolyCoeff{k,j},RXapprox{k,j});
        end
    end
end