function rout = MapCovarHermite(rin,funch1,funch2,mapCorr,calcReversion,M,xbound,zeropadLength)
% Utility for calculating the covariance mapping rX -> rY or the numerical 
% reversion of rY (which gives rX in a neighborhood of zero but might 
% diverge away from it):
%   rY = Cov(Y1,Y2) and rX = Cov(X1,X2) 
% with 
%   Y1=f1(X1) and Y2=f2(X2)
% for two functions f1 and f2 and X1, X2 bivariate N(0,1) variables
%
% The calculation of the mapping utilizes Hermite expansions.
%
% Usage:
%   rout = MapCovarHermite(rin,funch1,funch2,mapCorr,calcReversion,
%                          M,xbound,zeropadLength)
%   rout = MapCovarHermite(rin,funch1,funch2,mapCorr,calcReversion)
%   rout = MapCovarHermite(rin,funch1,funch2,mapCorr)
%   rout = MapCovarHermite(rin,funch1,funch2)
% Inputs:
%   rin       the covariance sequence to transform, see parameter
%             calcReversion
%   funch1, funch2  function handles for the functions f1 and f2 above
%   mapCorr   set to 1 to give the mapping rhoX->rhoY (or the numerical reversion for rhoY)
%             where rhoX = Cov(X1,X2)/sqrt(Var(X1)Var(X2)) 
%             (similar definition for rhoY)
%             Default value is 0
%   calcReversion  set to 1 for calculating the numerical reversion of rY
%             Default value is 0
%
%   The following parameters are optional and for fine-tuning of
%   the numerical calculation of the Hermite expansions used in the mapping
%   M         the number of terms in the Hermite expansion used for
%             calculating the mapping (see HermiteCoeffExpansion.m)
%   xbound    see HermiteCoeffExpansion.m
%   zeropadLength  see CovarSequence.m, 
%             Default value is 0
%
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon

if nargin < 6,
    M = 15; % number of terms in the Hermite expansion
    if nargin < 5,
        calcReversion = 0;
        if nargin < 4,
            mapCorr = 0;
        end
    end
end        

if ~exist('zeropadLength'),
    % zeropadLength 
	zeropadLength = 0; % the number of extra terms in the reversion series
end

if exist('xbound'),
    % xbound was provided
	coeffs1 = HermiteCoeffExpansion(funch1,M,xbound);
    coeffs2 = HermiteCoeffExpansion(funch2,M,xbound);
else
   % using default value of xbound
	coeffs1 = HermiteCoeffExpansion(funch1,M);
    coeffs2 = HermiteCoeffExpansion(funch2,M);
end
    
% when plotting the correlation mapping one needs to normalize
normFactor = 1;
if mapCorr
    % find the variance for each coordinate
    fwdSeq1 = CovarSequence(coeffs1, coeffs1, zeropadLength);
	fwdSeq2 = CovarSequence(coeffs2, coeffs2, zeropadLength);
    fwdPolyCoeff1 = fliplr([0 fwdSeq1]);
    fwdPolyCoeff2 = fliplr([0 fwdSeq2]);
    
	varY1 = polyval(fwdPolyCoeff1,1); % var for coordinate 1
    varY2 = polyval(fwdPolyCoeff2,1); % var for coordinate 2
    
    normFactor = 1/sqrt(varY1*varY2);
end

% normalize coefficients if requested
[fwdSeq, revSeq] = CovarSequence(coeffs1*normFactor, coeffs2, zeropadLength);

if calcReversion,
	% calculate reversion
    polyCoeff = fliplr([0 revSeq]); % setting constant term to 0 and flipping
    rout = polyval(polyCoeff,rin);
    rout = rout;
else
    % calculate forward mapping
    polyCoeff = fliplr([0 fwdSeq]); % setting constant term to 0 and flipping
    rout = polyval(polyCoeff,rin);
end