#include <math.h>
#define RELPREC 3.0e-14
#define MAXIT 12
#define PIM4 0.7511255444649425


/*  Evaluate the N-th order Hermite polynomial at a point x using a 
//  recurrence relation.
//  
//  Usage:
//  val = HermitePolyval(x,N,normalized)
//  
//  The Hermite polynomials are defined using the weight function w(x)=exp(-x^2/2)
//   
//  Returns H_N(x) if normalized==false or is omitted
//  Otherwise it returns \tilde{H}_N(x)=H_N(x)/sqrt(n!sqrt(2pi))
//  
//  Copyright 2009 (c), Hannes Helgason, ENS Lyon
//*/  
double evalHermitePoly(double, int, int);


/* This function is based on Chapter 4 in numerical Recipes in C:
// Given n, this routine returns arrays x[0...n-1], w[0...n-1] containing
// the abscissas and weights of the n-point Gauss-Hermite quadrature formula
// The largest abscissa is returned in x[0], the most negative in x[n-1]
//
// Copyright (c), Hannes Helgason, ENS Lyon, 2009
*/
void gaussHermiteQuadRule(double*, double*, int);


/*/////////////////////////////////////////////////////////////////////////////*/
/* IMPLEMENTATIONS */

double evalHermitePoly(double x, int N, int normalized) {
    double val;
    double Hnmin1, Hn, Hnplus1;
    int n;
    
    if (normalized==0) {
        if (N==0) {
            val = 1;
        } else {
            Hnmin1 = 0;
            Hn = 1;
            for (n=0; n<N; n++) {
                Hnplus1 = x*Hn - n*Hnmin1;
                Hnmin1 = Hn;
                Hn = Hnplus1;
            }
            val = Hnplus1;
        }
    } else {
        /* return normalized Hermite polynomial */
        if (N==0) {
            val = pow(2,-0.25)*PIM4;
        } else {
            Hnmin1 = 0;
            Hn = pow(2,-0.25)*PIM4;
            for (n=0; n<N; n++) {
                Hnplus1 = x/sqrt((double) (n+1)) * Hn - sqrt( ((double) n) / ((double) (n+1)) )*Hnmin1;
                Hnmin1 = Hn;
                Hn = Hnplus1;
            }
            val = Hnplus1;
        }
    }
    return val;
}


void gaussHermiteQuadRule(double *w, double *x, int n) {
    int k, its, j, m;
    double p1, p2, p3, pp, z, z1;
    
    m = (n+1)/2; /* note that this is integer division */
    /* roots are symmetric about the origin, only need to find half of them */
    for (k=1; k<=m; k++) {
        /* find k-th root
        / Initial guesses */
        if (k == 1) {
            z = sqrt((double) (2*n+1))-1.85575*pow((double) (2*n+1),-0.16667);
        } else if (k == 2) {
            z -= 1.14*pow((double) n, 0.426)/z;
        } else if (k == 3) {
            z = 1.86*z-0.86*x[0];
        } else if (k == 4) {
            z = 1.91*z-0.91*x[1];
        } else {
            z = 2.0*z-x[k-3];
        }
        /* Refinment of guess using Newton's method */
        for (its=1; its<=MAXIT; its++) {
            /* use recurrence relation to evaluate Hermite polynomial at z */
            p1 = PIM4;
            p2 = 0.0;
            for (j=1; j<=n; j++) {
                p3=p2;
                p2=p1;
                p1=z*sqrt(2.0/j)*p2-sqrt( ((double) (j-1))/j )*p3;
            }
            /* p1 is the Hermite polynomial
            // calculate its derivative */
            pp=sqrt((double)2*n)*p2;
            /* Newton's formula */
            z1=z;
            z=z1-p1/pp;
            if (fabs(z-z1)<=RELPREC) break;
        }
        /*if (iter>MAXIT) nrerror("(gaussHermiteQuadRule) Reached maximum number of iterations in root finding!");

        // store the root and its symmetric counterpart */
        x[k-1] = z;
        x[n-k] = -z;
        /* compute the weights */
        w[k-1] = 2.0/(pp*pp);
        w[n-k] = w[k-1];
	}
}
