/* 
 * On Windows, LAPACK symbols are not
 * exported with underscores e.g., zsteqr instead of zsteqr_
 */
#if !defined(_WIN32)
#define zsteqr zsteqr_
#define zhetrd zhetrd_
#define zungtr zungtr_
#endif

#include <mex.h>
#include <math.h>
#include "lapack.h"


/* Function for doing the Schur factorization step in the Gaussian synthesis algorithm.
 * Replaces negative eigenvalues with zeros if needed
 *
 * Implementing this in mex is orders of magnitude faster than using a for
 * loop in Matlab
 * 
 * Usage:
 *  [B,approxFlag, Lapprox] = FactLapack(L,P,M);
 * Inputs:
 *   L        auto- and cross-spectrum, complex matrix of size P*P-by-M
 *            L((k-1)*P + j,:) is spectrum for coordinate (k,j)
 * Outputs:
 *   B        matrix of size P-by-P*M where
 *            B(:,(m-1)*P+(1:P)) stores block Bm (of size P-by-P)
 *            Use as a input to the synthesis phase of the algorithm
 *   approxFlag   scalar which is equal to 0 if no approximation was needed.
 *            otherwise it is 1 if at least one eigenvalue in the Schur 
 *            factorizations was negative
 *   Lapprox  resulting auto- and cross-spectrum from setting negative eiganvalues in 
 *            the Schur factorizations to zero (if approxFlag=0, this should be equal to L)
 *            complex matrix of size P*P-by-M
 *            Lapprox((k-1)*P + j,:) is spectrum for coordinate (k,j)
 *
 * Copyright (c) 2010, Hannes Helgason, ENS de Lyon
 */


/* ZUNGTR( UPLO, N, A, LDA, TAU, WORK, LWORK, INFO )
 *      .. Scalar Arguments ..
 *      CHARACTER          UPLO
 *      INTEGER            INFO, LDA, LWORK, N
 *      .. Array Arguments ..
 *      COMPLEX*16         A( LDA, * ), TAU( * ), WORK( * )
 *  Purpose
 *  =======
 *
 *  ZUNGTR generates a complex unitary matrix Q which is defined as the
 *  product of n-1 elementary reflectors of order N, as returned by
 *  ZHETRD:
 *
 *  if UPLO = 'U', Q = H(n-1) . . . H(2) H(1),
 *
 *  if UPLO = 'L', Q = H(1) H(2) . . . H(n-1).
 *
 *  Arguments
 *  =========
 *
 *  UPLO    (input) CHARACTER*1
 *          = 'U': Upper triangle of A contains elementary reflectors
 *                 from ZHETRD;
 *          = 'L': Lower triangle of A contains elementary reflectors
 *                 from ZHETRD.
 *
 *  N       (input) INTEGER
 *          The order of the matrix Q. N >= 0.
 *
 *  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
 *          On entry, the vectors which define the elementary reflectors,
 *          as returned by ZHETRD.
 *          On exit, the N-by-N unitary matrix Q.
 *
 *  LDA     (input) INTEGER
 *          The leading dimension of the array A. LDA >= N.
 *
 *  TAU     (input) COMPLEX*16 array, dimension (N-1)
 *          TAU(i) must contain the scalar factor of the elementary
 *          reflector H(i), as returned by ZHETRD.
 *
 *  WORK    (workspace/output) COMPLEX*16 array, dimension (MAX(1,LWORK))
 *          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
 *
 *  LWORK   (input) INTEGER
 *          The dimension of the array WORK. LWORK >= N-1.
 *          For optimum performance LWORK >= (N-1)*NB, where NB is
 *          the optimal blocksize.
 *
 *          If LWORK = -1, then a workspace query is assumed; the routine
 *          only calculates the optimal size of the WORK array, returns
 *          this value as the first entry of the WORK array, and no error
 *          message related to LWORK is issued by XERBLA.
 *
 *  INFO    (output) INTEGER
 *          = 0:  successful exit
 *          < 0:  if INFO = -i, the i-th argument had an illegal value
 *
 */
extern void zungtr(
    char *uplo,
    int *n,
    double *a,
    int *lda,
    double *tau,
    double *work,
    int *lwork,
    int *info
);
 

/*    ZHETRD( UPLO, N, A, LDA, D, E, TAU, WORK, LWORK, INFO )
*     .. Scalar Arguments ..
*      CHARACTER          UPLO
*      INTEGER            INFO, LDA, LWORK, N
*     ..
*     .. Array Arguments ..
*      DOUBLE PRECISION   D( * ), E( * )
*      COMPLEX*16         A( LDA, * ), TAU( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  ZHETRD reduces a complex Hermitian matrix A to real symmetric
*  tridiagonal form T by a unitary similarity transformation:
*  Q**H * A * Q = T.
*
*  Arguments
*  =========
*
*  UPLO    (input) CHARACTER*1
*          = 'U':  Upper triangle of A is stored;
*          = 'L':  Lower triangle of A is stored.
*
*  N       (input) INTEGER
*          The order of the matrix A.  N >= 0.
*
*  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
*          On entry, the Hermitian matrix A.  If UPLO = 'U', the leading
*          N-by-N upper triangular part of A contains the upper
*          triangular part of the matrix A, and the strictly lower
*          triangular part of A is not referenced.  If UPLO = 'L', the
*          leading N-by-N lower triangular part of A contains the lower
*          triangular part of the matrix A, and the strictly upper
*          triangular part of A is not referenced.
*          On exit, if UPLO = 'U', the diagonal and first superdiagonal
*          of A are overwritten by the corresponding elements of the
*          tridiagonal matrix T, and the elements above the first
*          superdiagonal, with the array TAU, represent the unitary
*          matrix Q as a product of elementary reflectors; if UPLO
*          = 'L', the diagonal and first subdiagonal of A are over-
*          written by the corresponding elements of the tridiagonal
*          matrix T, and the elements below the first subdiagonal, with
*          the array TAU, represent the unitary matrix Q as a product
*          of elementary reflectors. See Further Details.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,N).
*
*  D       (output) DOUBLE PRECISION array, dimension (N)
*          The diagonal elements of the tridiagonal matrix T:
*          D(i) = A(i,i).
*
*  E       (output) DOUBLE PRECISION array, dimension (N-1)
*          The off-diagonal elements of the tridiagonal matrix T:
*          E(i) = A(i,i+1) if UPLO = 'U', E(i) = A(i+1,i) if UPLO = 'L'.
*
*  TAU     (output) COMPLEX*16 array, dimension (N-1)
*          The scalar factors of the elementary reflectors (see Further
*          Details).
*
*  WORK    (workspace/output) COMPLEX*16 array, dimension (MAX(1,LWORK))
*          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
*
*  LWORK   (input) INTEGER
*          The dimension of the array WORK.  LWORK >= 1.
*          For optimum performance LWORK >= N*NB, where NB is the
*          optimal blocksize.
*
*          If LWORK = -1, then a workspace query is assumed; the routine
*          only calculates the optimal size of the WORK array, returns
*          this value as the first entry of the WORK array, and no error
*          message related to LWORK is issued by XERBLA.
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
*  Further Details
*  ===============
*
*  If UPLO = 'U', the matrix Q is represented as a product of elementary
*  reflectors
*
*     Q = H(n-1) . . . H(2) H(1).
*
*  Each H(i) has the form
*
*     H(i) = I - tau * v * v'
*
*  where tau is a complex scalar, and v is a complex vector with
*  v(i+1:n) = 0 and v(i) = 1; v(1:i-1) is stored on exit in
*  A(1:i-1,i+1), and tau in TAU(i).
*
*  If UPLO = 'L', the matrix Q is represented as a product of elementary
*  reflectors
*
*     Q = H(1) H(2) . . . H(n-1).
*
*  Each H(i) has the form
*
*     H(i) = I - tau * v * v'
*
*  where tau is a complex scalar, and v is a complex vector with
*  v(1:i) = 0 and v(i+1) = 1; v(i+2:n) is stored on exit in A(i+2:n,i),
*  and tau in TAU(i).
*
*  The contents of A on exit are illustrated by the following examples
*  with n = 5:
*
*  if UPLO = 'U':                       if UPLO = 'L':
*
*    (  d   e   v2  v3  v4 )              (  d                  )
*    (      d   e   v3  v4 )              (  e   d              )
*    (          d   e   v4 )              (  v1  e   d          )
*    (              d   e  )              (  v1  v2  e   d      )
*    (                  d  )              (  v1  v2  v3  e   d  )
*
*  where d and e denote diagonal and off-diagonal elements of T, and vi
*  denotes an element of the vector defining H(i).
*
*  =====================================================================
*
*/
extern void zhetrd(
    char *uplo,
    int *n,
    double *a,
    int *lda,
    double *d,
    double *e,
    double *tau,
    double *work,
    int *lwork,
    int *info
);


/*    ZSTEQR( COMPZ, N, D, E, Z, LDZ, WORK, INFO )
*     .. Scalar Arguments ..
*      CHARACTER          COMPZ
*      INTEGER            INFO, LDZ, N
*     ..
*     .. Array Arguments ..
*      DOUBLE PRECISION   D( * ), E( * ), WORK( * )
*      COMPLEX*16         Z( LDZ, * )
*     ..
*
*  Purpose
*  =======
*
*  ZSTEQR computes all eigenvalues and, optionally, eigenvectors of a
*  symmetric tridiagonal matrix using the implicit QL or QR method.
*  The eigenvectors of a full or band complex Hermitian matrix can also
*  be found if ZHETRD or ZHPTRD or ZHBTRD has been used to reduce this
*  matrix to tridiagonal form.
*
*  Arguments
*  =========
*
*  COMPZ   (input) CHARACTER*1
*          = 'N':  Compute eigenvalues only.
*          = 'V':  Compute eigenvalues and eigenvectors of the original
*                  Hermitian matrix.  On entry, Z must contain the
*                  unitary matrix used to reduce the original matrix
*                  to tridiagonal form.
*          = 'I':  Compute eigenvalues and eigenvectors of the
*                  tridiagonal matrix.  Z is initialized to the identity
*                  matrix.
*
*  N       (input) INTEGER
*          The order of the matrix.  N >= 0.
*
*  D       (input/output) DOUBLE PRECISION array, dimension (N)
*          On entry, the diagonal elements of the tridiagonal matrix.
*          On exit, if INFO = 0, the eigenvalues in ascending order.
*
*  E       (input/output) DOUBLE PRECISION array, dimension (N-1)
*          On entry, the (n-1) subdiagonal elements of the tridiagonal
*          matrix.
*          On exit, E has been destroyed.
*
*  Z       (input/output) COMPLEX*16 array, dimension (LDZ, N)
*          On entry, if  COMPZ = 'V', then Z contains the unitary
*          matrix used in the reduction to tridiagonal form.
*          On exit, if INFO = 0, then if COMPZ = 'V', Z contains the
*          orthonormal eigenvectors of the original Hermitian matrix,
*          and if COMPZ = 'I', Z contains the orthonormal eigenvectors
*          of the symmetric tridiagonal matrix.
*          If COMPZ = 'N', then Z is not referenced.
*
*  LDZ     (input) INTEGER
*          The leading dimension of the array Z.  LDZ >= 1, and if
*          eigenvectors are desired, then  LDZ >= max(1,N).
*
*  WORK    (workspace) DOUBLE PRECISION array, dimension (max(1,2*N-2))
*          If COMPZ = 'N', then WORK is not referenced.
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*          > 0:  the algorithm has failed to find all the eigenvalues in
*                a total of 30*N iterations; if INFO = i, then i
*                elements of E have not converged to zero; on exit, D
*                and E contain the elements of a symmetric tridiagonal
*                matrix which is unitarily similar to the original
*                matrix.
*/
extern void zsteqr(
    char *compz,
    int *n,
    double *d,
    double *e,
    double *z,
    int *ldz,
    double *work,
    int *info
);

  
/* Mex-interface */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

    int nRowsL, nColsL;
	double *Br, *Bi; /* real and imaginary parts of B */
	double *Lr, *Li; /* real and imaginary parts of L */
    double *G; /* for storing the matrix to Schur factorize */
	double *Lapproxr, *Lapproxi; /* real and imaginary parts of Lapprox */
    int m, k, j, l, ind, M, P;
    double *approxFlag;
    int tmp, Psquared;
    double sqrtdj;
    
    /* declarations of variables used for FORTRAN routines (Schur factorization) */
	int n, lda, lwork, lworkGtr, lworkEqr, ldz;
    char uplo[2] = {'U','\0'};
    char compz[2] = {'V','\0'};
	char msg[101];
    double *d, *e, *tau, *work, *workQuery, *workGtr, *workEqr;
	int info;
    
    
    /* Get input data */
	if ((nlhs != 3) || (nrhs != 3)) {
        mexErrMsgTxt("Returning 3 output arguments and expecting 3 input arguments");
    }
    nRowsL = (int) mxGetM(prhs[0]);
    nColsL = (int) mxGetN(prhs[0]);
    P = (int) mxGetScalar(prhs[1]);
    M = (int) mxGetScalar(prhs[2]);
	Psquared = P*P;
    
    /*======= for FORTRAN routines ===========*/
	G = (double *) mxCalloc(2*Psquared, sizeof(double)); /* FORTRAN way of storing complex matrices */
    n = P;
    lda = P;
    
	/* initialize variables for zhetrd */
	e = (double *)mxCalloc(n-1,sizeof(double));
	d = (double *)mxCalloc(n,sizeof(double));
	tau = (double *)mxCalloc(2*(n-1),sizeof(double));    
	/* Query zhetrd on the value of lwork */
	lwork = -1;
	workQuery = (double *)mxCalloc(2,sizeof(double));
	zhetrd(uplo, &n, G, &lda, d, e, tau, workQuery, &lwork, &info);
	if (info < 0) {
        sprintf(msg, "Input %d to zhetrd had an illegal value",-info);
        mexErrMsgTxt(msg);
	} 
	/* now we have the value of lwork so we can initialize work */
	lwork = (int)(workQuery[0]);
	work = (double *)mxCalloc(2*lwork,sizeof(double));

    /* Query zungtr on the value of lworkGtr */
    lworkGtr = -1;
    zungtr(uplo, &n, G, &lda, tau, workQuery, &lworkGtr, &info);
    if (info < 0) {
        sprintf(msg, "Input %d to zungtr had an illegal value",-info);
        mexErrMsgTxt(msg);
    }
    /* now we have the value of lwork so we can initialize work */
    lworkGtr = (int)(workQuery[0]);
    lworkGtr = fmax(1,lworkGtr);
	workGtr = (double *)mxCalloc(lworkGtr,sizeof(double));
    
	/* initialize variables for zsteqr */
	ldz = n;
	lworkEqr = fmax(1,2*n-2);
	workEqr = (double *) mxCalloc(lworkEqr,sizeof(double));
    
    /*======= end for FORTRAN routines ===========*/
    
    if (nColsL != M) {
        mexErrMsgTxt("L should have M columns");
    }
	if (nRowsL != Psquared) {
        mexErrMsgTxt("L should have P*P rows");
    }
    /* get pointers to the data in L */
    Lr = mxGetPr(prhs[0]);
	Li = mxGetPi(prhs[0]);
    
    /* initialize output data */
	plhs[0] = mxCreateDoubleMatrix(P,P*M,mxCOMPLEX);
    Br = mxGetPr(plhs[0]);
    Bi = mxGetPi(plhs[0]);

    plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
    approxFlag = mxGetPr(plhs[1]);
    /* initialize flag */
    approxFlag[0] = 0;
    
	plhs[2] = mxCreateDoubleMatrix(Psquared,M,mxCOMPLEX);
	Lapproxr = mxGetPr(plhs[2]);
	Lapproxi = mxGetPi(plhs[2]);

    tmp = -Psquared; /* stores m*P*P */
	for(m=0; m<M; m++) {
        tmp += Psquared; /* calculate m*P*P */
        for(k=0; k<P; k++) {
            /* fill in entry (k,k) of G */
            ind = k*P + k;
            /* FORTRAN way of storing complex matrices */
            G[k*2*P + 2*k] = 2*Lr[tmp + ind];
            G[k*2*P + 2*k+1] = 0; /* imaginary part should be zero due to circulant embedding of auto-covariance */
            
            /* fill in upper-half of G (no need for lower-half) */
            for(j=k+1; j<P; j++) {
                ind = k*P + j;
                /* FORTRAN way of storing complex matrices */
                G[j*2*P + 2*k] = Lr[tmp + ind];
                G[j*2*P + 2*k+1] = Li[tmp + ind];
                /* PEND: 
                 * Multiplication by two is done in circembed, should be better
                 * to do here
                 */            
            }
        }
        
        
        /*
         * G could have negative eigenvalues.
         * Perform Schur decomposition of this Hermitian matrix
         * and construct the nonnegative definite matrix which best approximates
         * G under the Frobenius norm if needed
         */
        
        /*=================================================================*/
        /* Call LAPACK routine zhetrd: 
         * reduces a complex Hermitian matrix G to real symmetric
         * tridiagonal form by a unitary similarity transformation
         */
        /* call zhetrd */
        zhetrd(uplo, &n, G, &lda, d, e, tau, work, &lwork, &info);
        if (info < 0) {
            sprintf(msg, "Input %d to zhetrd had an illegal value",-info);
            mexErrMsgTxt(msg);
        }
        /* Call LAPACK routine zungtr to generate a complex unitary matrix Q 
         * defined as the product of the elementary reflectors 
         * as returned by zhetrd (stored in G and tau)
         */
        zungtr(uplo, &n, G, &lda, tau, workGtr, &lworkGtr, &info);
        if (info < 0) {
            sprintf(msg, "Input %d to zungtr had an illegal value",-info);
            mexErrMsgTxt(msg);
        }    
        /* the P-by-P unitary matrix is now stored in G */ 
        /* Call LAPACK routine zsteqr to compute eigenvalues and eigenvectors */
        zsteqr(compz, &n, d, e, G, &ldz, workEqr, &info);
        if (info < 0) {
            sprintf(msg, "Input %d to zsteqr had an illegal value",-info);
            mexErrMsgTxt(msg);
        }          
        /* At this point: 
         * the unitary matrix Q is stored in G
         * the eigenvalues are stored in d
         */        
        /*=================================================================*/
        
        /* check whether approximation is needed */
        for(k=0; k<P; k++) {
            if (d[k] < 0) {
                /* approximation is needed */
                approxFlag[0] = 1;
                d[k] = 0;
            }
        }
        /* 
         * Now we could construct the approximation Gmtilde = Q*diag(d)*(Q'), 
         * but instead we directly construct Bm in Gmtilde=Bm*Bm'
         * by Bm = Q*diag(sqrt(d))
         */
        for(j=0; j<P; j++) { 
            sqrtdj = sqrt(d[j]);
            for(k=0; k<P; k++) {
                Br[tmp + j*P + k] = sqrtdj*G[j*2*P + 2*k];
                Bi[tmp + j*P + k] = sqrtdj*G[j*2*P + 2*k+1];
            }
        }

       /* Calculate Gmtilde=Bm*Bm' and corresponding spectrum (Gmtilde/2)
        * to be able to get the resulting covariance sequence
        */
        for(j=0; j<P; j++) {
            for(k=0; k<P; k++) {
                ind = k*P + j;
                Lapproxr[tmp + ind] = 0;
                Lapproxi[tmp + ind] = 0;
                for (l=0; l<P; l++) {
                    Lapproxr[tmp + ind] += Br[tmp + l*P + k]*Br[tmp + l*P + j] + Bi[tmp + l*P + k]*Bi[tmp + l*P + j];
                    Lapproxi[tmp + ind] += -Br[tmp + l*P + k]*Bi[tmp + l*P + j] + Bi[tmp + l*P + k]*Br[tmp + l*P + j];
                }
                Lapproxr[tmp + ind] /= 2;
                Lapproxi[tmp + ind] /= 2;
            }
        }
        
     } /* end of for m */
    
        
    /* free memory */
	mxFree(G);
    
	/* free Fortran arrays */
	mxFree(e);
    mxFree(d);
    mxFree(tau);
    
    mxFree(workQuery);
    mxFree(work);
    mxFree(workEqr);
    mxFree(workGtr);
    
	
    return;
}